package ROTool::HTTP::Client;
use strict;
use ROTool;
use ROTool::Client;
use vars qw(@ISA);
use POSIX ":sys_wait_h";
#require 'jcode.pl';

my $APPRI_VER = "rotoold-httpd";

@ISA = qw( ROTool::Client ROTool );
my @field_list;

sub typechecker{
    #not a method.
    my $self = shift;
    if($self->{"buf"} =~ /HTTP\/\d+\.\d+/){
	bless $self;
	$self->new();
	return 1;
    }else{
	return 0;
    }
}

sub import{
    # don't delete above line.
    &ROTool::typechecker_regist(\&typechecker);
    
    @field_list = qw(id server area name hp hpmax sp spmax
		     baselv joblv class time message);
}

sub new{
    my $self = shift;
    # It's morph-processor method, not a constractor.
    $self = $self->SUPER::new(@_);
    $self->{"param"}->{"APPRI_VER"} = $APPRI_VER;
    return $self;
}

sub is_client{
    return 1;
}

sub is_server{
    return 0;
}

sub printlog{
    my $level   = shift;
    my $message = shift;
    &ROTool::printlog($level," HTTP module:".$message);
}

sub recv{
    my($self) = shift;

    my($ret) = $self->SUPER::recv();
    &printlog(4," HTTP module:[oid = $self->{oid}] parse message\n");

    my @client_msg = split(/(\r\n|\r|\n)/,$self->{"buf"});
    return -1 if(! @client_msg);
    while(@client_msg){
	$_ = shift(@client_msg);
	$_ .= shift(@client_msg) if(@client_msg);
	&ROTool::printlog(4,"[oid = $self->{oid}] parse line:{$_}\n");
	if(m{^(GET|HEAD)\s+(.*?)\s+HTTP/(\d+\.\d+)}i){
	    $self->{method} = $1;
	    $self->{path}   = $2;
	    $self->{httpVersion} = $3;
	    $self->{header} = {};
	    $self->{"MIME"} = "text/plain";
	    &ROTool::printlog(4,"  find:$1 $2 $3\n");
	}elsif(m{^(.*?):\s*(.*)$}){
	    $self->{header}->{lc($1)} = $2;
	    &ROTool::printlog(4,"  get HEADER [$1:$2]\n");
	}elsif(/^(\r\n|\r|\n)$/){
	    my $pid = fork();
	    if($pid){
		if(defined $self->{header}->{connection} &&
		   $self->{header}->{connection} =~ /close/i){
		    return -1;
		}
	    }else{
		my $sock = $self->{"sock"};

		&ROTool::printlog(4,"  send message\n");
		my $body;
		if($self->{method} =~ /HEAD/i){
		    $body = "";
		}elsif($self->{method} =~ /GET/i){
		    $body = $self->return_body();
		}
		my $head;
		
		$head  = "HTTP/1.1 200 OK\r\n";
		$head .= sprintf("Content-Type: %s; charset=EUC-JP\r\n",
				 $self->{"MIME"});
		$head .= "Content-Length: ".length($body)."\r\n";
		
		my $msg;
		if($body){
		    $msg = $head . "\r\n" . $body;
		}else{
		    $msg = $head . "\r\n";
		}
		
		# send message
		$sock->syswrite($msg,length($msg));
		&ROTool::printlog(4,$msg."\n");
		$self->{method} = undef;
		$self->{path}   = undef;
		exit();
	    }
	}else{
	    &ROTool::printlog(4,"  no match:$_\n");
	}
    }
    $self->{"buf"} = "";
    return $ret;
}
    
sub send{
    my $self = shift;
    return 1;
}

#sub return_body{
#    my $self = shift;
#    my $body = "Many of connecting clients : ".(scalar(&ROTool::UA_list()) - 1)."\r\n";
#
#    foreach my $client(&ROTool::UA_list()){
#	# Fetch data from UA_list.
#	# and make message send to peer client.
#	next if $client->{"param"}->{id} eq 'rotoold';
#	my $line;
#	$line .= sprintf("%-10s %-8s %-16s %-10s HP:%5s/%5s SP:%5s/%5s Lv:%2s/%2s %s %s %s\r\n",@{$client->{"param"}}{@field_list});
#	&jcode::convert(\$line,'euc');
#	$body .= $line;
#    }
#
#    return $body;
#}

sub print_entries{
    my $self = shift;
    my $body = "";
    foreach my $client(&ROTool::UA_list()){
	# Fetch data from UA_list.
	# and make message send to peer client.
	next if $client->{"param"}->{id} eq 'rotoold';
	my $line;
	$line .= sprintf("%-10s %-8s %-16s %-10s HP:%5s/%5s SP:%5s/%5s Lv:%2s/%2s %s %s %s<br>\r\n",@{$client->{"param"}}{@field_list});
	&jcode::convert(\$line,'euc');
	$body .= $line;
    }
    return $body;
}

sub print_entries_table{
    my $self = shift;
    my $body = "";
    foreach my $client(&ROTool::UA_list()){
	# Fetch data from UA_list.
	# and make message send to peer client.
	next if $client->{"param"}->{id} eq 'rotoold';
	my $line;
	$line .= "<tr>";
	foreach my $field_name (@field_list){
	    $line .=
		sprintf("<td class=gps_%s>%s</td>"
			,$field_name,$client->{"param"}->{$field_name});
	}
	$line .= "</tr>";
	&jcode::convert(\$line,'euc');
	$body .= $line;
    }
    return $body;
}

sub return_body{
    my $self = shift;
    my @path = ($self->{path});
    if($self->{path} =~ /^\/?\s*$/){
	@path = ("index.html","index.pl");
    }
    my $src;
    foreach (@path){
	my $path = "public_html/$_";
	&ROTool::printlog(4,"  check $path\n");
	if(-e "$path" && -f _){
	    open(FILE,$path);
	    while(<FILE>){
		$src .= $_;
	    }
	    close(FILE);
	    &ROTool::printlog(4,"  open $path\n");
	    if(-x _){
		$src = eval $src;
		$self->{"MIME"} = "text/html";
	    }else{
		$src =~ s/\%entries\%/$self->print_entries()/seg;
		$src =~ s/\%entries_table\%/$self->print_entries_table()/seg;
		$self->{"MIME"} = "text/html";
	    }
	    last;
	}
    }
    return $src;
}
1;

